'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var reactRedux = require('react-redux');
var redux = require('redux');
var __chunk_21 = require('./reducers/index.js');
var __chunk_28 = require('./core/exampleRangeResource.js');
var __chunk_29 = require('./core/exampleCache.js');
var __chunk_30 = require('./components/Viewport/index.js');
var __chunk_31 = require('./components/TransformScroller/index.js');
var __chunk_32 = require('./components/Ruler/index.js');
var __chunk_34 = require('./components/Plot/index.js');
var __chunk_35 = require('./components/Annotations/index.js');
var __chunk_36 = require('./components/SelectedAnnotation/index.js');
var __chunk_37 = require('./components/ChannelsList/index.js');
var __chunk_38 = require('./components/AnnotationsMinimap/index.js');
require('./reducers/navigation.js');
require('./actions/navigation.js');
require('./actions/presentation.js');
var __chunk_3 = require('./actions/header.js');
var __chunk_4 = require('./actions/data.js');
require('./actions/viewMode.js');
require('./actions/annotations.js');
require('./reducers/presentation.js');
require('./reducers/header.js');
require('./reducers/data.js');
require('./reducers/viewMode.js');
require('./reducers/annotations.js');
require('./workers/calculateCoordinates.js');
require('./workers/calculateDerived.js');
require('./selectors/viewMode.js');
require('reselect');
var __chunk_15 = require('./selectors/navigation.js');
require('./constants.js');
require('./selectors/annotations.js');
require('./core/edf/funnelBuffer.js');
require('./core/edf/edfHeader.js');
require('./core/edf/edfStreamReader.js');
require('./core/edf/edf.js');
require('./core/edf/arrBuffer.js');
require('./core/deferred.js');
require('./components/Plot/SegmentCanvas.js');

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Array equality helper function.
// Nothing fancy, no support for nested arrays (we don't need it).
function equalArrays(arr1, arr2) {
  var i = arr1.length;
  if (i !== arr2.length) {
    return false;
  }

  while (i--) {
    if (arr1[i] !== arr2[i]) {
      return false;
    }
  }

  return true;
}

var Studio = function (_Component) {
  _inherits(Studio, _Component);

  function Studio(props) {
    _classCallCheck(this, Studio);

    var _this = _possibleConstructorReturn(this, (Studio.__proto__ || Object.getPrototypeOf(Studio)).call(this, props));

    _this.handleSegmentChange = function () {
      var forceRequest = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      if (!_this.store) {
        console.warn('Trying to update when store is unavailable; <Studio> component likely unmounted.');
        return;
      }

      if (!_this.cached) {
        console.warn('Cache not initialized yet; ignoring store update.');
        return;
      }

      var newSegmentRange = __chunk_15.getCurrentSegmentRange(_this.store.getState());

      if (forceRequest !== true && equalArrays(newSegmentRange, _this.currentSegmentRange)) {
        return;
      }

      _this.currentSegmentRange = newSegmentRange;
      _this.cached.getChunksAsync(30 * _this.currentSegmentRange[0], 30 * _this.currentSegmentRange.length).then(function (data) {
        // Check - segment might have changed (e.g. due to user navigation)
        // while we were fetching data from cache. In this case, discard data
        // altogether; there was more recent cache call, it will put proper
        // data to the store.
        //
        // This is not optimal - we might arrange to put to store those parts
        // of data which *are* in the current segment range. But in this case
        // deletion becomes a huge problem, so we avoid this, until much
        // more robust solution is designed.
        var newerSegmentRange = __chunk_15.getCurrentSegmentRange(_this.store.getState());
        if (!equalArrays(newerSegmentRange, _this.currentSegmentRange)) {
          console.warn('Segment has changed while getting data; data discarded');
          return;
        }

        var _this$store$getState = _this.store.getState(),
            rawValues = _this$store$getState.data.rawValues;

        for (var i = 0; i < rawValues.length; i++) {
          var iInData = _this.currentSegmentRange.indexOf(i);
          if (iInData !== -1 && !rawValues[i]) {
            // Add segment to store if we received it from resource/cache and
            // don't have it in store already...
            _this.store.dispatch(__chunk_4.addDataSegment30s(i, data.slice(30 * iInData, 30 * iInData + 30)));
          } else if (iInData === -1 && rawValues[i]) {
            // ...otherwise, remove from store. That's essentially garbage
            // collecting.
            _this.store.dispatch(__chunk_4.removeDataSegment30s(i));
          }
        }
      }); // Omitting component update - should we?
    };

    if (props.store) {
      props.store.replaceReducer(__chunk_21.default);
      _this.store = props.store;
    } else {
      _this.store = redux.createStore(__chunk_21.default);
    }
    _this.currentSegmentRange = __chunk_15.getCurrentSegmentRange(_this.store.getState());
    _this.unsubscribeFromStore = null;
    _this.cached = null;
    return _this;
  }

  // handleSegmentChange() implementation depends on the assumption that it's not
  // debounced or throttled.


  _createClass(Studio, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      this.unsubscribeFromStore = this.store.subscribe(this.handleSegmentChange);

      var edfResource = new __chunk_28.default({
        url: this.props.url
      });

      edfResource.fetchHeader().then(function (header) {
        // 1. Get / set header and init cache
        if (process.env.NODE_ENV === 'development') {
          console.log('App:header', header);
        }
        _this2.cached = new __chunk_29.default(edfResource, header);
        _this2.store.dispatch(__chunk_3.setHeader(header));
      }).then(function () {
        // 2. Ask for whatever data we need
        _this2.handleSegmentChange(true);
      });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.cached && this.cached.destroy();
      this.cached = null;

      this.unsubscribeFromStore();
      this.store = null;
    }
  }, {
    key: 'render',
    value: function render() {
      return React__default.createElement(
        reactRedux.Provider,
        { store: this.store },
        React__default.createElement(
          'div',
          { className: 'eegStudio' },
          React__default.createElement(
            __chunk_31.default,
            null,
            React__default.createElement(__chunk_32.default, null),
            React__default.createElement(
              __chunk_30.default,
              null,
              React__default.createElement(__chunk_35.default, null),
              React__default.createElement(__chunk_34.default, null),
              React__default.createElement(__chunk_36.default, null)
            )
          ),
          React__default.createElement(__chunk_37.default, null),
          React__default.createElement(__chunk_38.default, null)
        )
      );
    }
  }]);

  return Studio;
}(React.Component);

module.exports = Studio;
//# sourceMappingURL=Studio.js.map
