import React, { Component } from 'react';
import { Provider } from 'react-redux';
import { createStore } from 'redux';
import rootReducer from './reducers/index.js';
import EDFResource from './core/exampleRangeResource.js';
import Cache from './core/exampleCache.js';
import Viewport from './components/Viewport/index.js';
import Scroller from './components/TransformScroller/index.js';
import Ruler from './components/Ruler/index.js';
import Plot from './components/Plot/index.js';
import Annotations from './components/Annotations/index.js';
import SelectedAnnotation from './components/SelectedAnnotation/index.js';
import ChannelsList from './components/ChannelsList/index.js';
import AnnotationsMinimap from './components/AnnotationsMinimap/index.js';
import './reducers/navigation.js';
import './actions/navigation.js';
import './actions/presentation.js';
import { setHeader } from './actions/header.js';
import { addDataSegment30s, removeDataSegment30s } from './actions/data.js';
import './actions/viewMode.js';
import './actions/annotations.js';
import './reducers/presentation.js';
import './reducers/header.js';
import './reducers/data.js';
import './reducers/viewMode.js';
import './reducers/annotations.js';
import './workers/calculateCoordinates.js';
import './workers/calculateDerived.js';
import './selectors/viewMode.js';
import 'reselect';
import { getCurrentSegmentRange } from './selectors/navigation.js';
import './constants.js';
import './selectors/annotations.js';
import './core/edf/funnelBuffer.js';
import './core/edf/edfHeader.js';
import './core/edf/edfStreamReader.js';
import './core/edf/edf.js';
import './core/edf/arrBuffer.js';
import './core/deferred.js';
import './components/Plot/SegmentCanvas.js';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Array equality helper function.
// Nothing fancy, no support for nested arrays (we don't need it).
function equalArrays(arr1, arr2) {
  var i = arr1.length;
  if (i !== arr2.length) {
    return false;
  }

  while (i--) {
    if (arr1[i] !== arr2[i]) {
      return false;
    }
  }

  return true;
}

var Studio = function (_Component) {
  _inherits(Studio, _Component);

  function Studio(props) {
    _classCallCheck(this, Studio);

    var _this = _possibleConstructorReturn(this, (Studio.__proto__ || Object.getPrototypeOf(Studio)).call(this, props));

    _this.handleSegmentChange = function () {
      var forceRequest = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      if (!_this.store) {
        console.warn('Trying to update when store is unavailable; <Studio> component likely unmounted.');
        return;
      }

      if (!_this.cached) {
        console.warn('Cache not initialized yet; ignoring store update.');
        return;
      }

      var newSegmentRange = getCurrentSegmentRange(_this.store.getState());

      if (forceRequest !== true && equalArrays(newSegmentRange, _this.currentSegmentRange)) {
        return;
      }

      _this.currentSegmentRange = newSegmentRange;
      _this.cached.getChunksAsync(30 * _this.currentSegmentRange[0], 30 * _this.currentSegmentRange.length).then(function (data) {
        // Check - segment might have changed (e.g. due to user navigation)
        // while we were fetching data from cache. In this case, discard data
        // altogether; there was more recent cache call, it will put proper
        // data to the store.
        //
        // This is not optimal - we might arrange to put to store those parts
        // of data which *are* in the current segment range. But in this case
        // deletion becomes a huge problem, so we avoid this, until much
        // more robust solution is designed.
        var newerSegmentRange = getCurrentSegmentRange(_this.store.getState());
        if (!equalArrays(newerSegmentRange, _this.currentSegmentRange)) {
          console.warn('Segment has changed while getting data; data discarded');
          return;
        }

        var _this$store$getState = _this.store.getState(),
            rawValues = _this$store$getState.data.rawValues;

        for (var i = 0; i < rawValues.length; i++) {
          var iInData = _this.currentSegmentRange.indexOf(i);
          if (iInData !== -1 && !rawValues[i]) {
            // Add segment to store if we received it from resource/cache and
            // don't have it in store already...
            _this.store.dispatch(addDataSegment30s(i, data.slice(30 * iInData, 30 * iInData + 30)));
          } else if (iInData === -1 && rawValues[i]) {
            // ...otherwise, remove from store. That's essentially garbage
            // collecting.
            _this.store.dispatch(removeDataSegment30s(i));
          }
        }
      }); // Omitting component update - should we?
    };

    if (props.store) {
      props.store.replaceReducer(rootReducer);
      _this.store = props.store;
    } else {
      _this.store = createStore(rootReducer);
    }
    _this.currentSegmentRange = getCurrentSegmentRange(_this.store.getState());
    _this.unsubscribeFromStore = null;
    _this.cached = null;
    return _this;
  }

  // handleSegmentChange() implementation depends on the assumption that it's not
  // debounced or throttled.


  _createClass(Studio, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      this.unsubscribeFromStore = this.store.subscribe(this.handleSegmentChange);

      var edfResource = new EDFResource({
        url: this.props.url
      });

      edfResource.fetchHeader().then(function (header) {
        // 1. Get / set header and init cache
        if (process.env.NODE_ENV === 'development') {
          console.log('App:header', header);
        }
        _this2.cached = new Cache(edfResource, header);
        _this2.store.dispatch(setHeader(header));
      }).then(function () {
        // 2. Ask for whatever data we need
        _this2.handleSegmentChange(true);
      });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.cached && this.cached.destroy();
      this.cached = null;

      this.unsubscribeFromStore();
      this.store = null;
    }
  }, {
    key: 'render',
    value: function render() {
      return React.createElement(
        Provider,
        { store: this.store },
        React.createElement(
          'div',
          { className: 'eegStudio' },
          React.createElement(
            Scroller,
            null,
            React.createElement(Ruler, null),
            React.createElement(
              Viewport,
              null,
              React.createElement(Annotations, null),
              React.createElement(Plot, null),
              React.createElement(SelectedAnnotation, null)
            )
          ),
          React.createElement(ChannelsList, null),
          React.createElement(AnnotationsMinimap, null)
        )
      );
    }
  }]);

  return Studio;
}(Component);

export default Studio;
//# sourceMappingURL=Studio.js.map
